//genesis
/* 
** This simulation script performs 4 simulations :
** 1 : TEA/TTX blocked mitral cell model, inj=0.62nA
** 2 : TEA/TTX blocked mitral cell model, no Ca in soma/axons.
** 3 : Unblocked mitral cell model, inj=0.5nA
** 4 : Unblocked mitral cell model, no trode leak in soma.
*/

/* Simulation parameters */
float dt = 50e-6
float filet = 100e-6
setclock 0 {dt}
setclock 1 {filet}
float pre_stim_time=0.05
float post_stim_time=0.12
float simtime = 0.32
float injblock = 0.62e-9
float injnoblock = 0.5e-9
str refblock="correct.block"
str refnoblock="correct.noblock"
str mitblock="P53.p"
str mitnoblock="mit.p"

/* Display_mode selects how you want the output.
** 0 : Only file output
** 1 : Xodus output
** 2 : xplot output
*/
int display_mode = 1

/*
** hines_mode is a flag to select the level of optimization for the
** hsolver.
** Level 0 does not use the hines solver.
** Level 1 sets the hines solver to the older (slower) mode.
** Level 2 is the most optimized.
*/
int	hines_mode = 2

/*
** Loading in the prototypes
*/
if (!{exists(/library)})
		include defaults
		include compartments
		include bulbchan2
	
		ce /library
	
		make_cylind_compartment
		make_LCa3_mit_usb
    	make_K2_mit_usb
    	make_K_mit_usb
		make_Na_rat_smsnn
		make_KA_bsg_yka
		set KA_bsg_yka Ek -0.07
		make_KM_bsg_upi
		set KM_bsg_upi Ek -0.07
		make_Na_mit_usb
		make_Na2_mit_usb
		make_Ca_mit_conc
		make_Kca_mit_usb
		ce /
end


/*
** Creating the cells 
*/
// Reading in the basic cell descriptor files 
read_cell {mitnoblock} /mit
/* We could read in the 'blocked' model, which would be more efficient.
** instead, let us do the channel blocking by reducing the conductances
** on the complete model.
** read_cell {mitblock} /block
*/
copy /mit /block
// Eliminate Na, K2, and reduce K appropriately
scale_parms(/block,/mit, \
	#[]/Na_mit_usb,Gbar,0.0, \
	#[]/K2_mit_usb,Gbar,0.0,\
	#[]/K_mit_usb,Gbar,{1.0/1.3})


// Duplicating the unblocked model and eliminating electrode leak
float correct_Rm=RM/(32e-6*32e-6*PI)
copy /mit /noleak
set /noleak/soma Rm {correct_Rm}

// duplicating the blocked model and eliminating Ca from the
// soma,axon and secondary dends, and doubling it in the glom
copy /block /noca
set /noca/soma/LCa3_mit_usb Gbar 0.0
set /noca/axon[]/LCa3_mit_usb Gbar 0.0
// Doubling the conc in the glom to compensate
scale_parms(/noca,/block,glom#[]/LCa3_mit_usb,Gbar,2.0)

// Setting the injection currents
// We need to scale the noleak input current to the input resistance
// Note that this is not an exact equivalent, since the electrotonic
// structure of the cell has changed considerably
float mit_input_res,noleak_input_res
	mit_input_res=calc_Rm(/mit/soma)
	noleak_input_res=calc_Rm(/noleak/soma)
create neutral /stim
set /stim x {injnoblock} 
set /stim y {1.6*injnoblock*mit_input_res/noleak_input_res}
set /stim z {injblock}
set /mit/soma inject {injnoblock} 

sendmsg /stim /mit/soma INJECT x
sendmsg /stim /noleak/soma INJECT y
sendmsg /stim /block/soma INJECT z
sendmsg /stim /noca/soma INJECT z

// Setting up the hsolvers
if (hines_mode > 0)
	create hsolve /mit/solve
	set ^ path /mit/##[][TYPE=compartment]
	create hsolve /noleak/solve
	set ^ path /noleak/##[][TYPE=compartment]
	create hsolve /block/solve
	set ^ path /block/##[][TYPE=compartment]
	create hsolve /noca/solve
	set ^ path /noca/##[][TYPE=compartment]

	if (hines_mode == 1)
		set /#/solve comptmode 1 chanmode 1 useconcen 1
	end
	if (hines_mode == 2)
		set /#/solve comptmode 1 chanmode 3 useconcen 1
	end
	call /mit/solve SETUP
	call /noleak/solve SETUP
	call /block/solve SETUP
	call /noca/solve SETUP
	setmethod 11
end

// Setting up the file I/O
create asc_file /output/mitVm
create asc_file /output/noleakVm
create asc_file /output/blockVm
create asc_file /output/nocaVm
sendmsg /mit/soma /output/mitVm SAVE Vm
sendmsg /noleak/soma /output/noleakVm SAVE Vm
sendmsg /block/soma /output/blockVm SAVE Vm
sendmsg /noca/soma /output/nocaVm SAVE Vm
set	/output/# leave_open 1 initialize 1 flush 0
useclock /output/# 1

// Seting up graphics
if (display_mode==1)
	xstartup
	create xform /form [0,0,1000,800]
	create xgraph /form/mit [50%,0%,50%,50%]
	create xgraph /form/noleak [50%,50%,50%,50%]
	create xgraph /form/block [0%,0%,50%,50%]
	create xgraph /form/noca [0%,50%,50%,50%]
	set /form/# ymin -0.1 ymax 0.05 xmax {pre_stim_time+simtime+post_stim_time}
	sendmsg /mit/soma /form/mit PLOT Vm *somaVm *red
	sendmsg /noleak/soma /form/noleak PLOT Vm *somaVm *green
	sendmsg /block/soma /form/block PLOT Vm *somaVm *blue
	sendmsg /noca/soma /form/noca PLOT Vm *somaVm *orange
	xshow /form
	useclock /form/# 1
end

// Running simulation
reset
set /stim x 0 y 0 z 0
step -t {pre_stim_time}
set /stim x {injnoblock} 
set /stim y {1.6*injnoblock*mit_input_res/noleak_input_res}
set /stim z {injblock}
step -t {simtime}
set /stim x 0 y 0 z 0
step -t {post_stim_time}

// Flushing pending file output
set /output/# flush 1 leave_open 0
call /output/# PROCESS

// Displaying xplots in mode 2
if (display_mode==2)
	plotall
	/*
	xplot correct.block -geometry =300x300+0+0 &
	xplot correct.noblock -geometry =300x300+300+0 &
	xplot blockVm -geometry =300x300+0+300 &
	sleep 2
	xplot mitVm -geometry =300x300+300+300 &
	sleep 2
	xplot nocaVm -geometry =300x300+0+600 &
	sleep 2
	xplot noleakVm -geometry =300x300+300+600 &
	*/
end

if (display_mode != 1)
	quit
end
